#! /usr/bin/python

"""//Web service to bill credits spent for a batch//
 
- File: bill.py

- Arguments - Using the HTTP GET method:
 - batch_id -- The batch identifier, as a string
 - credit -- The number of credits spent, as a positive integer
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">True</div>`` if the bill is accepted
 - ``<div class="qos">False</div>`` if the bill is denied
 
- Notes: 
  - The bill will be denied if all the credits allocated to a batch's order have already been spent.
  - A database configuration file named "db.cfg" must be present in the directory.

"""


import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)



def bill(conn,batch_id,credit):
    """Function implementing the  bill operation

    Arguments:
    conn -- The CreditSystem's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
    credit -- The number of credits spent, as a positive integer
    
    The function returns True if the bill is accepted, or False otherwise

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None          
    if not isinstance(credit,int) or credit<=0:
        log.error("The credit argument must be a positive integer !")
        return None
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None
        
    c=conn.cursor()

    log.info("Retrieving order for batch identifier %s..." % batch_id)
    try:
        c.execute("SELECT credit_balance FROM QoSOrder WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None     

    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such order associated to this batch identifier in the database")
        return None
    elif len(res)>1:
        log.error("Several orders with batch identifier %s in the database ! 'batch_id' should be the table PRIMARY KEY"% batch_id)
        return None
    
    try:    
        order_credit_balance = int(res[0][0])
        if order_credit_balance<0:
            raise ValueError("Negative credit_balance")
    except ValueError as e:        
        log.error("'credit_balance' is not a positive or zero integer in the database")  
        log.error(e)
        return None
    
    log.info("Order found, credit balance is %d" % order_credit_balance)
    if order_credit_balance<credit:
        log.warning("This batch order has not enough credit for this bill, rejecting bill...")
        return False
    else:
        log.info("Updating credit balance...")
        try:
            c.execute("UPDATE QoSOrder SET credit_balance=%d WHERE batch_id='%s'" % (order_credit_balance-credit,batch_id))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None 
        log.info("Done! New credit balance is %d" % (order_credit_balance-credit))
        return True
    

if __name__=="__main__":
    cgi_func.print_header()

    if (cgi_func.get_form_value('batch_id') is None) or (cgi_func.get_form_value('credit') is None) :
        cgi_func.print_form(['batch_id','credit'])
    else:
        batch_id = cgi_func.get_form_value('batch_id')
        try:
            credit = int(cgi_func.get_form_value('credit'))
        except ValueError as e:
            log.error("credit argument must be an integer")
            cgi_func.print_form(['batch_id','credit'])
        else:
            conn=db_func.connect(db_func.read_db_config("db.cfg"))
        
            cgi_func.print_result( bill(conn,batch_id,credit))
        
            db_func.disconnect(conn)

    cgi_func.print_footer()
