#! /usr/bin/python

"""//Tools to grab information from a plain file and store it into the QoSInfo database//

"""

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)


def grab(conn,filename,from_timestamp=None,to_timestamp=None):
    """Grab information from a plain file and insert it into the database. It reads from file with following line structure: 
    ```
    <timestamp> <batch_id> <number of completed jobs> <number of uncompleted jobs>
    ```
    and inserts the corresponding information in the QoSInfo database table "BatchCompletionHistory". ``<timestamp>`` denotes the number of seconds since 1970-01-01 00:00:00 UTC. 

    - File: grabber_plainfile.py
         
    - Arguments:
     - conn -- The Info's DB connection, as a MySQL.connections.Connection object
     - filename -- The name of the file containing the information, as a string
     - from_timestamp (optional) -- Only grab information with date after this argument, as a number of seconds since 1970-01-01 00:00:00 UTC.
     - to_timestamp (optional) -- Only grab information with date prior to this argument, as a number of seconds since 1970-01-01 00:00:00 UTC.
         
    """    
    if from_timestamp is not None and (not isinstance(from_timestamp,int) or from_timestamp<0):
        log.error("The from_timestamp argument must be None or a positive integer !")
        return None
    if to_timestamp is not None and (not isinstance(to_timestamp,int) or to_timestamp<0):
        log.error("The to_timestamp argument must be None or a positive integer !")
        return None
    if from_timestamp is not None and to_timestamp is not None and from_timestamp>to_timestamp:
        log.error("The from_timestamp argument must be lower than to_timestamp argument !")
        return None        
    
    c=conn.cursor()
    
    log.info("Grabbing information on plain textfile")
    try:
        f = open(filename)
    except IOError as e:
        log.error("Enable to open file %s" % filename)
        log.error(e)
    else:
        for l in f:
            data=l.split()
            try:
                ts=int(data[0])
                batch_id = data[1]
                n_compl=int(data[2])
                n_ucompl=int(data[3])
            except TypeError as e:
                log.error("Line contains some invalid data structure : %s" % l)
                log.error(e)
            else :
                if ((from_timestamp is None) or (ts>from_timestamp)) and ((to_timestamp is None) or (ts<to_timestamp)):
                    log.info("Inserting line in database : %s" % l)
                    c.execute("INSERT INTO BatchCompletionHistory VALUES (%d,'%s',%d,%d)" % (ts,batch_id,n_compl,n_ucompl))
    log.info("Information grabbing completed")




if __name__=="__main__":
    if len(sys.argv)>1:
        trace_file=sys.argv[1]
    else:
        trace_file="example.tr"
    conn=db_func.connect(db_func.read_db_config("db.cfg"))
    grab(conn,trace_file)
    db_func.disconnect(conn)

