#! /usr/bin/python

"""Unit tests for calculate_cloud_benefit.py

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
sys.path.append(os.path.join(sys.path[0],"..",".."))
os.chdir(os.path.join(sys.path[0],".."))

from tools import db_func
from get_elapsed_time import get_elapsed_time
import unittest
import random
import datetime
import time 

DEFAULT_BATCH_ID = "batch_0"

class BadArgument(unittest.TestCase):
    """Test for bad arguments

    
    """
    
    def testUnknownBatchId(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        batch_id = str(random.randint(0,1000))        
        self.assertEqual(get_elapsed_time(conn,batch_id), None)
        db_func.disconnect(conn)

    def testNonStrBatchId(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        batch_id = random.randint(0,1000)
        self.assertEqual(get_elapsed_time(conn,batch_id), None)
        db_func.disconnect(conn)

    def testWrongDBConn(self):
        conn=db_func.connect(db_func.read_db_config("../scheduler/db.cfg"))
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID), None)
        db_func.disconnect(conn)

    def testNoDBConn(self):
        self.assertEqual(get_elapsed_time(None,DEFAULT_BATCH_ID), None)

    def testNegativeCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        current_time = -random.randint(0,1000)
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,current_time), None)
        db_func.disconnect(conn)
        
    def testNonIntCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        current_time = str(random.randint(0,1000))
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,current_time), None)
        db_func.disconnect(conn)
        
    def testNonBoolUseLastDBEntry(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,use_last_db_entry="Toto"), None)
        db_func.disconnect(conn)
        

class BadReturn(unittest.TestCase):
    """Test for bad returned values
    
    """
    
    def testReturnArePositiveIntType(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))    
        self.assert_(isinstance(get_elapsed_time(conn,DEFAULT_BATCH_ID), int))        
        self.assert_(isinstance(get_elapsed_time(conn,DEFAULT_BATCH_ID,1290176137), int))        
        self.assert_(isinstance(get_elapsed_time(conn,DEFAULT_BATCH_ID,use_last_db_entry=True), int))        
        self.assert_(get_elapsed_time(conn,DEFAULT_BATCH_ID)>0)        
        self.assert_(get_elapsed_time(conn,DEFAULT_BATCH_ID,1290176137)>0)        
        self.assert_(get_elapsed_time(conn,DEFAULT_BATCH_ID,use_last_db_entry=True)>0)        
        db_func.disconnect(conn)


class ConsistencyCheck(unittest.TestCase):
    """Test consistency
    
    """

    def testElapsedTimeAndCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))        
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID), get_elapsed_time(conn,DEFAULT_BATCH_ID,int(time.mktime(datetime.datetime.utcnow().timetuple()))))
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,1290176137), get_elapsed_time(conn,DEFAULT_BATCH_ID,1290000000)+176137)
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,1290176137), get_elapsed_time(conn,DEFAULT_BATCH_ID)-int(time.mktime(datetime.datetime.utcnow().timetuple()))+1290176137)
        db_func.disconnect(conn)   
   
    def testUseLastDBEntryOverCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))        
        self.assertEqual(get_elapsed_time(conn,DEFAULT_BATCH_ID,123456,use_last_db_entry=True), get_elapsed_time(conn,DEFAULT_BATCH_ID,use_last_db_entry=True))
        db_func.disconnect(conn)   
    
    
    
    
            
if __name__ == "__main__":
    unittest.main()   
