#! /usr/bin/python

"""Unit tests for calculate_cloud_benefit.py

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
sys.path.append(os.path.join(sys.path[0],"..",".."))
os.chdir(os.path.join(sys.path[0],".."))

from tools import db_func
from get_percent_completed import get_percent_completed
import unittest
import random
import datetime
import time 

DEFAULT_BATCH_ID = "batch_0"

class BadArgument(unittest.TestCase):
    """Test for bad arguments

    
    """
    
    def testUnknownBatchId(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        batch_id = str(random.randint(0,1000))        
        self.assertEqual(get_percent_completed(conn,batch_id), None)
        db_func.disconnect(conn)

    def testNonStrBatchId(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        batch_id = random.randint(0,1000)
        self.assertEqual(get_percent_completed(conn,batch_id), None)
        db_func.disconnect(conn)

    def testWrongDBConn(self):
        conn=db_func.connect(db_func.read_db_config("../scheduler/db.cfg"))
        self.assertEqual(get_percent_completed(conn,DEFAULT_BATCH_ID), None)
        db_func.disconnect(conn)

    def testNoDBConn(self):
        self.assertEqual(get_percent_completed(None,DEFAULT_BATCH_ID), None)

    def testNegativeCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        current_time = -random.randint(0,1000)
        self.assertEqual(get_percent_completed(conn,DEFAULT_BATCH_ID,current_time), None)
        db_func.disconnect(conn)
        
    def testNonIntCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
        current_time = str(random.randint(0,1000))
        self.assertEqual(get_percent_completed(conn,DEFAULT_BATCH_ID,current_time), None)
        db_func.disconnect(conn)
        

class BadReturn(unittest.TestCase):
    """Test for bad returned values
    
    """
    
    def testReturnAreFloatBetweenZeroAndOne(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))    
        self.assert_(isinstance(get_percent_completed(conn,DEFAULT_BATCH_ID), float))        
        self.assert_(isinstance(get_percent_completed(conn,DEFAULT_BATCH_ID,1290176137), float))        
        self.assert_(get_percent_completed(conn,DEFAULT_BATCH_ID)>=0 and get_percent_completed(conn,DEFAULT_BATCH_ID)<=1)        
        self.assert_(get_percent_completed(conn,DEFAULT_BATCH_ID,1290176137)>=0 and get_percent_completed(conn,DEFAULT_BATCH_ID,1290176137)<=1)        
        db_func.disconnect(conn)


class ConsistencyCheck(unittest.TestCase):
    """Test consistency
    
    """

    def testPercentageCompletedAndCurrentTime(self):
        conn=db_func.connect(db_func.read_db_config("info_db.cfg"))        
        self.assertEqual(get_percent_completed(conn,DEFAULT_BATCH_ID), get_percent_completed(conn,DEFAULT_BATCH_ID,int(time.mktime(datetime.datetime.utcnow().timetuple()))))
        db_func.disconnect(conn)   
   
    
    
    
            
if __name__ == "__main__":
    unittest.main()   
