#! /usr/bin/python

"""Unit tests for calculate_cloud_benefit.py

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
sys.path.append(os.path.join(sys.path[0],"..",".."))
os.chdir(os.path.join(sys.path[0],".."))

from tools import db_func
from start_qos import start_qos
import unittest
import random


class BadArgument(unittest.TestCase):
    """Test for bad arguments

    
    """

    def testNonStrArg(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        grid_tech = random.randint(0,1000)        
        self.assertEqual(start_qos(conn,grid_tech,"localhost"), None)
        grid_host = random.randint(0,1000)        
        self.assertEqual(start_qos(conn,"BOINC",grid_host), None)
        db_func.disconnect(conn)

    def testWrongDBConn(self):
        conn=db_func.connect(db_func.read_db_config("../info/db.cfg"))
        self.assertEqual(start_qos(conn,"BOINC","localhost"), None)
        db_func.disconnect(conn)

    def testNoDBConn(self):
        self.assertEqual(start_qos(None,"BOINC","localhost"), None)


class BadReturn(unittest.TestCase):
    """Test for bad returned values
    
    """
    
    def testReturnIsString(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))    
        self.assert_(isinstance(start_qos(conn,"BOINC","localhost"), str))        
        db_func.disconnect(conn)


class ConsistencyCheck(unittest.TestCase):
    """Test consistency
    
    """

    def testNotTheSameBatchId(self):
        conn=db_func.connect(db_func.read_db_config("db.cfg"))        
        self.assert_(start_qos(conn,"BOINC","localhost") != start_qos(conn,"BOINC","localhost") )
        db_func.disconnect(conn)   
   
    
    
    
            
if __name__ == "__main__":
    unittest.main()   
